# **NOTE** from @dawid 2021-06-21:
#   This example (likely) doesn't work.
#   PPO has been tested and proven to work on environments with a single agent
#   so the suspicion is that there is a bug in MultiEnvRunner or PPO processing multi agents.


import os
from typing import List

import pylab as plt

from ai_traineree.agents.ppo import PPOAgent as Agent

# from ai_traineree.loggers import NeptuneLogger
from ai_traineree.loggers import TensorboardLogger
from ai_traineree.runners.env_runner import MultiSyncEnvRunner
from ai_traineree.tasks import GymTask
from ai_traineree.types import TaskType

neptune_api_key = os.getenv("NEPTUNE_API_TOKEN")

env_name = "LunarLanderContinuous-v2"
# data_logger = NeptuneLogger("kretyn/PPO-LunarLander-Multi", api_token=neptune_api_key)
# data_logger= None
data_logger = TensorboardLogger()
processes = 1
num_workers = processes
kwargs = {
    "device": "cuda",
    "num_workers": num_workers,
    "rollout_length": 1000,
    "batch_size": 1000,
    "number_updates": 80,
    "using_gae": True,  # Default is True
    "hidden_layers": (64, 64),
    "ppo_ratio_clip": 0.1,
    "entropy_weight": 0.005,
    "critic_lr": 1e-3,
    "actor_lr": 3e-4,
    "max_grad_norm_actor": 20.0,
    "max_grad_norm_critic": 20.0,
    "target_kl": 0.02,
}
tasks: List[TaskType] = [GymTask(env_name) for _ in range(num_workers)]
agent = Agent(tasks[0].obs_space, tasks[0].action_space, **kwargs)
env_runner = MultiSyncEnvRunner(tasks, agent, processes=processes, data_logger=data_logger)
scores = env_runner.run(reward_goal=80, eps_end=0.01, max_episodes=5000, force_new=True, log_episode_freq=10)

# plot the scores
fig = plt.figure()
ax = fig.add_subplot(111)
plt.plot(range(len(scores)), scores)
plt.ylabel("Score")
plt.xlabel("Episode #")
plt.savefig(f"{env_name}.png", dpi=120)
plt.show()



import pylab as plt

from ai_traineree.agents.ppo import PPOAgent as Agent
from ai_traineree.loggers import TensorboardLogger
from ai_traineree.runners.env_runner import EnvRunner
from ai_traineree.tasks import GymTask
from ai_traineree.types import TaskType

env_name = "LunarLanderContinuous-v2"
data_logger = TensorboardLogger()
task: TaskType = GymTask(env_name)
config = {
    "rollout_length": 500,
    "batch_size": 500,
    "number_updates": 60,
    "hidden_layers": (64, 64),
    "using_gae": True,  # Default is True
    "ppo_ratio_clip": 0.1,
    "entropy_weight": 0.05,
    "action_scale": 1,
    "max_grad_norm_actor": 10.0,
    "max_grad_norm_critic": 10.0,
    "critic_lr": 1e-4,
    "actor_lr": 3e-4,
    "using_kl_div": True,
    "std_min": 0,
    "std_max": 10,
    "std_init": 0.6,
    "simple_policy": True,
}
agent = Agent(task.obs_space, task.action_space, **config)
env_runner = EnvRunner(task, agent, data_logger=data_logger)
# env_runner.interact_episode(eps=0, render=True)
scores = env_runner.run(100, 2000, eps_decay=0.9, eps_end=0.001, force_new=True, log_episode_freq=10)
env_runner.interact_episode(eps=0, render=True)

data_logger.close()

# plot the scores
fig = plt.figure()
ax = fig.add_subplot(111)
plt.plot(range(len(scores)), scores)
plt.ylabel("Score")
plt.xlabel("Episode #")
plt.savefig(f"{env_name}.png", dpi=120)
plt.show()
